"use strict";
/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Michal Malý <michal.maly@ibt.cas.cz>
 * @author Jiří Černý <jiri.cerny@ibt.cas.cz>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfalPyramidsColorThemeProvider = exports.ConfalPyramidsColorTheme = exports.getConfalPyramidsColorThemeParams = exports.ConfalPyramidsColorThemeParams = void 0;
var tslib_1 = require("tslib");
var color_1 = require("../color");
var property_1 = require("./property");
var types_1 = require("./types");
var property_2 = require("../property");
var color_2 = require("../../../mol-theme/color");
var color_3 = require("../../../mol-util/color");
var params_1 = require("../../../mol-util/color/params");
var param_definition_1 = require("../../../mol-util/param-definition");
var legend_1 = require("../../../mol-util/legend");
var type_helpers_1 = require("../../../mol-util/type-helpers");
var Description = 'Assigns colors to confal pyramids';
var PyramidsColors = (0, color_3.ColorMap)(tslib_1.__assign({}, color_1.NtCColors));
exports.ConfalPyramidsColorThemeParams = {
    colors: param_definition_1.ParamDefinition.MappedStatic('default', {
        'default': param_definition_1.ParamDefinition.EmptyGroup(),
        'custom': param_definition_1.ParamDefinition.Group((0, params_1.getColorMapParams)(PyramidsColors))
    }),
};
function getConfalPyramidsColorThemeParams(ctx) {
    return param_definition_1.ParamDefinition.clone(exports.ConfalPyramidsColorThemeParams);
}
exports.getConfalPyramidsColorThemeParams = getConfalPyramidsColorThemeParams;
function ConfalPyramidsColorTheme(ctx, props) {
    var colorMap = props.colors.name === 'default' ? PyramidsColors : props.colors.params;
    function color(location, isSecondary) {
        var _a;
        if (types_1.ConfalPyramidsTypes.isLocation(location)) {
            var _b = location.data, step = _b.step, isLower = _b.isLower;
            var key = step.NtC + "_".concat(isLower ? 'Lwr' : 'Upr');
            return (_a = colorMap[key]) !== null && _a !== void 0 ? _a : color_1.ErrorColor;
        }
        return color_1.ErrorColor;
    }
    return {
        factory: ConfalPyramidsColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: Description,
        legend: (0, legend_1.TableLegend)((0, type_helpers_1.ObjectKeys)(colorMap).map(function (k) { return [k.replace('_', ' '), colorMap[k]]; }).concat([['Error', color_1.ErrorColor]])),
    };
}
exports.ConfalPyramidsColorTheme = ConfalPyramidsColorTheme;
exports.ConfalPyramidsColorThemeProvider = {
    name: 'confal-pyramids',
    label: 'Confal Pyramids',
    category: color_2.ColorTheme.Category.Residue,
    factory: ConfalPyramidsColorTheme,
    getParams: getConfalPyramidsColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.ConfalPyramidsColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure && ctx.structure.models.some(function (m) { return property_2.Dnatco.isApplicable(m); }); },
    ensureCustomProperties: {
        attach: function (ctx, data) { return data.structure ? property_1.ConfalPyramidsProvider.attach(ctx, data.structure.models[0], void 0, true) : Promise.resolve(); },
        detach: function (data) { return data.structure && property_1.ConfalPyramidsProvider.ref(data.structure.models[0], false); }
    }
};
