"use strict";
/**
 * Copyright (c) 2017-2022 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.CIF = void 0;
var tslib_1 = require("tslib");
var parser_1 = require("./cif/text/parser");
var parser_2 = require("./cif/binary/parser");
var schema_1 = require("./cif/schema");
var mmcif_1 = require("./cif/schema/mmcif");
var ccd_1 = require("./cif/schema/ccd");
var bird_1 = require("./cif/schema/bird");
var dic_1 = require("./cif/schema/dic");
var density_server_1 = require("./cif/schema/density-server");
var cif_core_1 = require("./cif/schema/cif-core");
var segmentation_1 = require("./cif/schema/segmentation");
exports.CIF = {
    parse: function (data) { return typeof data === 'string' ? (0, parser_1.parseCifText)(data) : (0, parser_2.parseCifBinary)(data); },
    parseText: parser_1.parseCifText,
    parseBinary: parser_2.parseCifBinary,
    toDatabaseCollection: schema_1.toDatabaseCollection,
    toDatabase: schema_1.toDatabase,
    schema: {
        mmCIF: function (frame) { return (0, schema_1.toDatabase)(mmcif_1.mmCIF_Schema, frame); },
        CCD: function (frame) { return (0, schema_1.toDatabase)(ccd_1.CCD_Schema, frame); },
        BIRD: function (frame) { return (0, schema_1.toDatabase)(bird_1.BIRD_Schema, frame); },
        dic: function (frame) { return (0, schema_1.toDatabase)(dic_1.dic_Schema, frame); },
        cifCore: function (frame) { return (0, schema_1.toDatabase)(cif_core_1.CifCore_Schema, frame, cif_core_1.CifCore_Aliases); },
        densityServer: function (frame) { return (0, schema_1.toDatabase)(density_server_1.DensityServer_Data_Schema, frame); },
        segmentation: function (frame) { return (0, schema_1.toDatabase)(segmentation_1.Segmentation_Data_Schema, frame); },
    }
};
tslib_1.__exportStar(require("./cif/data-model"), exports);
