"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OverlayTaskProgress = exports.BackgroundTaskProgress = void 0;
var tslib_1 = require("tslib");
var jsx_runtime_1 = require("react/jsx-runtime");
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var base_1 = require("./base");
var immutable_1 = require("immutable");
var common_1 = require("./controls/common");
var icons_1 = require("./controls/icons");
var react_1 = require("react");
var use_behavior_1 = require("./hooks/use-behavior");
function BackgroundTaskProgress() {
    var plugin = (0, react_1.useContext)(base_1.PluginReactContext);
    var _a = (0, react_1.useState)((0, immutable_1.OrderedMap)()), tracked = _a[0], setTracked = _a[1];
    (0, react_1.useEffect)(function () {
        var started = plugin.events.task.progress.subscribe(function (e) {
            var _a;
            var hideOverlay = !!((_a = plugin.spec.components) === null || _a === void 0 ? void 0 : _a.hideTaskOverlay);
            if (e.level === 'background' && (hideOverlay || !e.useOverlay)) {
                setTracked(function (tracked) { return tracked.set(e.id, e); });
            }
        });
        var finished = plugin.events.task.finished.subscribe(function (_a) {
            var id = _a.id;
            setTracked(function (tracked) { return tracked.delete(id); });
        });
        return function () {
            started.unsubscribe();
            finished.unsubscribe();
        };
    }, [plugin]);
    return (0, jsx_runtime_1.jsxs)("div", { className: 'msp-background-tasks', children: [tracked.valueSeq().map(function (e) { return (0, jsx_runtime_1.jsx)(ProgressEntry, { event: e }, e.id); }), (0, jsx_runtime_1.jsx)(CanvasCommitState, {})] });
}
exports.BackgroundTaskProgress = BackgroundTaskProgress;
function CanvasCommitState() {
    var _a;
    var plugin = (0, react_1.useContext)(base_1.PluginReactContext);
    var queueSize = (0, use_behavior_1.useBehavior)((_a = plugin.canvas3d) === null || _a === void 0 ? void 0 : _a.commitQueueSize);
    if (!queueSize)
        return null;
    return (0, jsx_runtime_1.jsx)("div", { className: 'msp-task-state', children: (0, jsx_runtime_1.jsx)("div", { children: (0, jsx_runtime_1.jsxs)("div", { children: ["Commiting renderables... ", queueSize, " remaining"] }) }) });
}
var ProgressEntry = /** @class */ (function (_super) {
    tslib_1.__extends(ProgressEntry, _super);
    function ProgressEntry() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.abort = function () {
            _this.plugin.managers.task.requestAbort(_this.props.event.progress.root.progress.taskId, 'User Request');
        };
        return _this;
    }
    ProgressEntry.prototype.render = function () {
        var root = this.props.event.progress.root;
        var subtaskCount = countSubtasks(this.props.event.progress.root) - 1;
        var pr = root.progress.isIndeterminate
            ? void 0
            : (0, jsx_runtime_1.jsxs)(jsx_runtime_1.Fragment, { children: ["[", root.progress.current, "/", root.progress.max, "]"] });
        var subtasks = subtaskCount > 0
            ? (0, jsx_runtime_1.jsxs)(jsx_runtime_1.Fragment, { children: ["[", subtaskCount, " subtask(s)]"] })
            : void 0;
        return (0, jsx_runtime_1.jsx)("div", { className: 'msp-task-state', children: (0, jsx_runtime_1.jsxs)("div", { children: [root.progress.canAbort && (0, jsx_runtime_1.jsx)(common_1.IconButton, { svg: icons_1.CancelSvg, onClick: this.abort, title: 'Abort' }), (0, jsx_runtime_1.jsxs)("div", { children: [root.progress.message, " ", pr, " ", subtasks] })] }) });
    };
    return ProgressEntry;
}(base_1.PluginUIComponent));
function countSubtasks(progress) {
    if (progress.children.length === 0)
        return 1;
    var sum = 0;
    for (var _i = 0, _a = progress.children; _i < _a.length; _i++) {
        var c = _a[_i];
        sum += countSubtasks(c);
    }
    return sum;
}
function OverlayTaskProgress() {
    var plugin = (0, react_1.useContext)(base_1.PluginReactContext);
    var _a = (0, react_1.useState)((0, immutable_1.OrderedMap)()), tracked = _a[0], setTracked = _a[1];
    (0, react_1.useEffect)(function () {
        var started = plugin.events.task.progress.subscribe(function (e) {
            if (!!e.useOverlay) {
                setTracked(function (tracked) { return tracked.set(e.id, e); });
            }
        });
        var finished = plugin.events.task.finished.subscribe(function (_a) {
            var id = _a.id;
            setTracked(function (tracked) { return tracked.delete(id); });
        });
        return function () {
            started.unsubscribe();
            finished.unsubscribe();
        };
    }, [plugin]);
    if (tracked.size === 0)
        return null;
    return (0, jsx_runtime_1.jsx)("div", { className: 'msp-overlay-tasks', children: tracked.valueSeq().map(function (e) { return (0, jsx_runtime_1.jsx)(ProgressEntry, { event: e }, e.id); }) });
}
exports.OverlayTaskProgress = OverlayTaskProgress;
